//
// blogparser.cpp
//
// @author Ka Cheung Sia {kcsia@cs.ucla.edu}
// @date   02/01/2005
// @copyright Ka Cheung Sia
//
// Implementation of a processing routine to convert BLOG HTML to RSS
//
//
// Copyright (c) 2005, Ka Cheung Sia {kcsia@cs.ucla.edu}
// University of California, Los Angeles
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of the University of California Los Angeles nor
// the names of its contributors may be used to endorse or promote
// products derived from this software without specific prior written
// permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "ricBLOGParser.h"
#include <stdio.h>
#include <string>
#include <iostream>
#include <map>
#include <fstream>

using namespace kcsia;


int main (int argc, char* argv[]) {
    FILE *fin;
    char input_file[MAX_FILE_SIZE];
    char **blog_text;
    Timestamp *blog_time;
    int num_entries=0;
    ricBLOGParser ricblogparser;

    memset (input_file,0,sizeof(char)*MAX_FILE_SIZE);

    if (argc == 2 ) {
      // reading from a single file or from stdin
      fin=fopen(argv[1],"r");
      fread(input_file,sizeof(char),MAX_FILE_SIZE,fin);
      fclose(fin);
      
      // parse the blog
      num_entries=ricblogparser.parseBlog(input_file,&blog_text,&blog_time);

      if (num_entries) {
        ricblogparser.printRSS(argv[1],input_file,num_entries,blog_text,blog_time);
      }
      
      // clean up
      for (int i=0;i<num_entries;i++)
        if (NULL != blog_text[i]) {
          free(blog_text[i]);
        }
      if (num_entries) {
        free(blog_text);
        free(blog_time);
      }
    } else if (argc == 4) {
      // more than 1 input parameter, batch mode

      std::map<int,string> urlmap;
      std::ifstream f_urllist(argv[1]);
      if (!f_urllist) {
        cerr << "can't open url list file" << argv[1] << endl;
        return 0;
      }
      string url;
      int id=0;
      do {
        f_urllist >> url;
        urlmap[id++]=url;
      } while (!f_urllist.eof());
      f_urllist.close();
      
      cout << "Read the URL list" << endl;
      
      for (int i=0;i<id;i++) {
        // initialize global variables before every parse
        memset(input_file,0,sizeof(char)*MAX_FILE_SIZE);
        
        // try read in the html file
        string infilename(argv[2]);
        char fileid_str[30];
        sprintf(fileid_str,"%d",i);
        if ('/'==infilename[infilename.length()-1])
          infilename.append(fileid_str);
        else {
          infilename.append("/");
          infilename.append(fileid_str);
        }
        
        FILE *in_fd=fopen(infilename.c_str(),"r");
        if (in_fd==NULL) {
          cout << "File not exist, skipping: " << infilename << endl;
          continue;
        }
        
        fread(input_file,sizeof(char),MAX_FILE_SIZE,in_fd);
        fclose(in_fd);
        
        cout << "Read: " << infilename << endl;
        
        num_entries=ricblogparser.parseBlog(input_file,&blog_text,&blog_time);
        
        if (num_entries) {
          cout << "Contains recognizable weblog entries" << endl;
          
          // prepare to write RSS to output file
          string outfilename(argv[3]);
          if ('/'==outfilename[outfilename.length()-1])
            outfilename.append(fileid_str);
          else {
            outfilename.append("/");
            outfilename.append(fileid_str);
          }
          
          std::ofstream outRSS(outfilename.c_str());
          if (!outRSS) {
            cout << "Error opening file " << outfilename << "to write" << endl;
          } else {
            ricblogparser.printRSS(urlmap[i], input_file, num_entries, blog_text, blog_time, outRSS);
            cout << "finish converting to RSS: " << outfilename << endl;
          }
          
          
          for (int i=0;i<num_entries;i++)
            if (NULL != blog_text[i]) {
              free(blog_text[i]);
            }
          
          if (num_entries) {
            free(blog_text);
            free(blog_time);
          }
          num_entries=0;
          blog_text=NULL;
          blog_time=NULL;
          
        }    
      } 
    } else {
      // print usage
      cout << "Usage:\n";
      cout << argv[0] << " <input HTML filename>  or\n";
      cout << argv[0] << " <URL list> <input directory> <output directory>\n";
    }
    return 0;    
}


