//
// ricBLOGParser.cpp
//
// @author Ka Cheung Sia {kcsia@cs.ucla.edu}
// @date   02/01/2005
// @copyright Ka Cheung Sia
//
// Implementation of a parser for blog
//
//
// Copyright (c) 2005, Ka Cheung Sia {kcsia@cs.ucla.edu}
// University of California, Los Angeles
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of the University of California Los Angeles nor
// the names of its contributors may be used to endorse or promote
// products derived from this software without specific prior written
// permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include "alxHTMLParser.h"
#include "ricBLOGParser.h"
#include <boost/regex.hpp>
#include <string>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <queue>
#include <stack>
// HTML Tidy Header file
//#include <tidy.h>
//#include <buffio.h>
//#include <errno.h>



using namespace kcsia;

// static memeber variable in ricBLOGParser
int ricBLOGParser::instance_count=0;
Node ricBLOGParser::root;
Node* ricBLOGParser::current=&ricBLOGParser::root;
stack<int> ricBLOGParser::tag_stack;
char ricBLOGParser::buffer[MAX_FILE_SIZE];
char *ricBLOGParser::ptr_buffer=ricBLOGParser::buffer;

// constructor
ricBLOGParser::ricBLOGParser () {
    
    // maintain Singleton
    if (instance_count) {
        cerr << "Can't instantiate more than once" << endl;
        return;
    } else
        instance_count++;

    // initialize static member vairables
    memset(buffer,0,sizeof(char)*MAX_FILE_SIZE);
    memset(&root,0,sizeof(Node));
    ptr_buffer=buffer;
    root.ID=0;
    root.start=ptr_buffer;
    root.end=ptr_buffer;
    current=&root;
    
    // initialize memeber variables
    timestampentries=NULL;
    blogentries=NULL;
    blogentries_count=0;

    // set the call back functions to use static class memeber function
    setOpenTagHandler  (&ricOpenTagHandler);
    setCloseTagHandler (&ricCloseTagHandler);
    setTokenHandler    (&ricTokenHandler);
    setCommentHandler  (&ricCommentHandler);
    setPunctHandler    (&ricPunctHandler);
    setUnmatchedHandler(&ricUnmatchedHandler);
    setEntityHandler   (&ricEntityHandler);

    // call a function to install the default regualr expressions into this parser
    initializeDateRE();
}

// destructor
ricBLOGParser::~ricBLOGParser () {
    // free the regular expression objects allocated in initializeRE function
    for (int i=0; i < date_re; i++)
        delete date_re_engine[i];
    for (int i=0; i < time_re; i++)
        delete time_re_engine[i];

    instance_count--;
}


// the function to parse a blog
int ricBLOGParser::parseBlog (char* input, char*** blog_text, Timestamp** blog_time) {

    char tempbuffer[MAX_FILE_SIZE];

    // initialize static memeber variables for use in parsing
    memset(buffer,0,sizeof(char)*MAX_FILE_SIZE);
    memset(&root,0,sizeof(Node));
    ptr_buffer=buffer;
    root.ID=0;
    root.start=ptr_buffer;
    root.end=ptr_buffer;
    current=&root;
    
    // initialize memeber variables
    timestampentries=NULL;
    blogentries=NULL;
    blogentries_count=0;


    /*
    // use HTML tidy to clean the input first
    TidyBuffer output = {0};
    TidyBuffer errbuf = {0};
    int rc = -1;
    Bool ok;
    
    TidyDoc tdoc = tidyCreate();                     // Initialize "document"
    
    ok = tidyOptSetBool( tdoc, TidyXhtmlOut, yes );  // Convert to XHTML
    if ( ok )
      rc = tidySetErrorBuffer( tdoc, &errbuf );      // Capture diagnostics
    if ( rc >= 0 )
      rc = tidyParseString( tdoc, input );           // Parse the input
    if ( rc >= 0 )
      rc = tidyCleanAndRepair( tdoc );               // Tidy it up!
    if ( rc >= 0 )
      rc = tidyRunDiagnostics( tdoc );               // Kvetch
    if ( rc > 1 ) {
      // error in tidy
      strncpy(tempbuffer,input,strlen(input));
    } else if ( rc >= 0 ) {
      // success in tidy
      rc = tidySaveBuffer( tdoc, &output );          // Pretty Print
      strncpy(tempbuffer,(char*)output.bp,strlen((char*)output.bp));
    }
    
    tidyBufFree( &output );
    tidyBufFree( &errbuf );
    tidyRelease( tdoc );
    //////////////////////////////////////////////////////////////
    */

    // initialize parser to use the input string, parse and clean up
    useString(input);
    while (html_lex() != ELEM_EOF);  // building of DOM tree
    deleteCurrentBuf();
    
    // need to handle the case when final stack size is not zero
    if (tag_stack.size() != 0) {
        handleDOMCloseTag(0,NULL,0,tag_stack.size());
    }

    //while (current != &root) {
    //    current->end=ptr_buffer;
    //    current = current->parent;
    //}
    
    if (current != & root) {

      cerr << "[ricBLOGParser] Serious error, ending is not root\n";

    } else {
    
      // do BFS traversal (parsing) will store stuff in
      // blogentries_count, blogentries, timestampentries
      // return 1 if successful, 0 if fail
      DOMParseBFS(&root);
    
      // assign results to output
      *blog_text=blogentries;
      *blog_time=timestampentries;
      blogentries=NULL;
      timestampentries=NULL;
    }

    // clean up
    for (int i=0;i<root.num_children;i++)
        freeDOM(root.children[i]);
    if (root.num_children)
        free(root.children);
    // erase all stuff in tag_stack
    while (!tag_stack.empty())
        tag_stack.pop();
    
    
    return blogentries_count;
    
}


void ricBLOGParser::freeDOM (Node *node) {
    for (int i=0;i<node->num_children;i++)
        freeDOM (node->children[i]);
    if (0!=node->num_children && node->children != NULL)
        free(node->children);
    if (node != NULL)
        free(node);
}


/**
 * when we encounter an open tag, what to do
 */
int ricBLOGParser::ricOpenTagHandler (int id, char* tag, int len) {
    switch (id) {
    case ELEM_A:
    case ELEM_BODY:
    case ELEM_B:
    case ELEM_DIV:
    case ELEM_FRAME:
    case ELEM_FRAMESET:
    case ELEM_FONT:
      //    case ELEM_FORM:
    case ELEM_HEAD:
    case ELEM_HTML:
    case ELEM_IFRAME:
    case ELEM_P:
    case ELEM_TABLE:
    case ELEM_TBODY:
    case ELEM_TR:
    case ELEM_TD:
      //    case ELEM_OL:
      //    case ELEM_UL:
      //    case ELEM_LI:
    case ELEM_H1:
    case ELEM_H2:
    case ELEM_H3:
    case ELEM_H4:
    case ELEM_H5:
    case ELEM_H6:
    case ELEM_EM:
    case ELEM_BLOCKQUOTE:
    case ELEM_SPAN:
        tag_stack.push(id);
        handleDOMOpenTag(id, tag, len);
        break;
    default:
        //  call a function to stuff the things inside
        handleDOMNormalString(id, tag, len);
        break;
    }
    return 1;
}

/**
 * when we encounter a close tag, what to do
 */
int ricBLOGParser::ricCloseTagHandler (int id, char* tag, int len) {
    bool done=false;
    // this is a temporaray stack to store the skipped open tag
    stack<int> tmp_stack;

    // when nothing opens, but receive a close tag, simply ignore it and treat it as normal string
    if (tag_stack.size()<1) {
        handleDOMNormalString(id, tag, len);
        return 1;
    }

    do {
        // top of the stack
        int val=tag_stack.top();

        switch (id-ELEM_CLOSE) {
        case ELEM_A:
        case ELEM_BODY:
        case ELEM_B:
        case ELEM_DIV:
        case ELEM_FRAME:
        case ELEM_FRAMESET:
        case ELEM_FONT:
          //        case ELEM_FORM:
        case ELEM_HEAD:
        case ELEM_HTML:
        case ELEM_IFRAME:
        case ELEM_P:
        case ELEM_TABLE:
        case ELEM_TBODY:
        case ELEM_TR:
        case ELEM_TD:
          //        case ELEM_OL:
          //        case ELEM_UL:
          //        case ELEM_LI:
        case ELEM_H1:
        case ELEM_H2:
        case ELEM_H3:
        case ELEM_H4:
        case ELEM_H5:
        case ELEM_H6:
        case ELEM_EM:
        case ELEM_BLOCKQUOTE:
        case ELEM_SPAN:
            // for those closing tags of our interset, we check if they matches opening tag
            if (val != id-ELEM_CLOSE ) {
                if ( tag_stack.size() <= 1 || val == ELEM_BODY || val == ELEM_HEAD || val == ELEM_HTML) {
                    // it is seriously mismatched, skip this closing tag
                    // just treat this close tag as oridinary string
#ifdef DEBUG
                    std::cout << "the stack will be empty, so must skip close tag\n";
                    std::cout << "skipped close tag is" << id << endl;
#endif

                    // this is subtle, when we skip open tags to find matches for close tag
                    // might run into this case (which we choose to skip the close tag instead)
                    // then we need to push back those skipped open tags back
                    while (tmp_stack.size() != 0) {
                        tag_stack.push(tmp_stack.top());
                        tmp_stack.pop();
                    };
                    handleDOMNormalString(id, tag, len);
                    done=true;
                } else {
                    if (tmp_stack.size() == 9) {
                        // after 11 trials already, need to skip closing tag
                        // just treat this close tag as oridinary string, so we
                        // push back those skipped open tags back to the original stack
                        while (tmp_stack.size() != 0) {
                            tag_stack.push(tmp_stack.top());
                            tmp_stack.pop();
                        };
#ifdef DEBUG
                        std::cout << "give up matching closing tag: " << id << endl;
#endif
                        handleDOMNormalString(id, tag, len);
                        done=true;
                    } else {
                        // keep skipping open tag to see if there is a match
                        // don't need to handle the close tag right now
                        tmp_stack.push(val);
                        tag_stack.pop();

#ifdef DEBUG
                        std::cout << "try skipping open tag: " << val << endl;
#endif
                    }
                }
            } else {
                // matched
#ifdef DEBUG
                std::cout << "poping " << id << endl;
#endif
                tag_stack.pop();
                handleDOMCloseTag (id, tag, len, tmp_stack.size());
                if (tmp_stack.size() != 0) {
                    // should we push back the tags ? or just leave it here? which is better?
#ifdef DEBUG
                    std::cout << "matched after skipping " << tmp_stack.size() << " open tags\n";
#endif
                    while (!tmp_stack.empty())
                        tmp_stack.pop();
                    //tmp.clear();
                    // do {
                    //     tag_stack.push_back(tmp.back());
                    //     tmp.pop_back();
                    // } while (tmp.size() != 0);
                }
                done=true;
            }
            break;
        default:
            // need to call function to stuff things inside though
            done=true;
            handleDOMNormalString(id, tag, len);
            break;
        }
    } while (!done);    

    return 1;

}

int ricBLOGParser::ricTokenHandler (int id, char* tok, int len) {
    // some words, just stuff in the DOM tree
    handleDOMNormalString(id, tok, len);
    return 1;
}

int ricBLOGParser::ricEntityHandler (int ent) {
    // like &lt;
    // printf ("entity: %d\n",ent);
    return 1;
}

int ricBLOGParser::ricPunctHandler (int id, char* punct, int len) {
    // printf ("punctutation:  %d, %s\n", id, punct);
    handleDOMNormalString(id, punct, len);
    return 1;
}

int ricBLOGParser::ricUnmatchedHandler (int id, char* str, int len) {
    // just ignore it, don't put into the DOM tree
    // can we skip this? seems most of the time are \n character
    handleDOMNormalString(id, str, len);
    return 1;
}

int ricBLOGParser::ricCommentHandler (int id, char* comment, int len) {
    // just ignore it, don't put into the DOM tree
    // can skip recording this down
    return 1;
}

/**
 * go one level deeper in the DOM tree
 */
int ricBLOGParser::handleDOMOpenTag (int id, char* str, int len) {
    // create a new child and make the current node point to this new child
    Node* new_node=(Node*)calloc(1,sizeof(Node));
    // can I skip some of the initialization when i zero out the memory?
    // memset((void*)new_node,0,sizeof(Node));

    new_node->ID=id;
    new_node->start=ptr_buffer;
    // assign the parent to currently pointing node
    new_node->parent=current;
    
    // copy content to buffer
    strncpy(ptr_buffer,str,len);
    ptr_buffer+=len;

    // building the tree
    current->children=(Node**)realloc((void*)current->children,sizeof(Node*)*(current->num_children+1));
    current->children[current->num_children]=new_node;
    if (current->num_children)
        current->children[current->num_children-1]->next_sibling=new_node;
    current->num_children=current->num_children+1;

    current=new_node;

    return 0;
}


/**
 * go n level shallower up the DOM tree
 */
int ricBLOGParser::handleDOMCloseTag (int id, char* str, int len, int level_up) {
    // level up is to trace back to how many level up to close that tag
    strncpy(ptr_buffer,str,len);

    current->end=ptr_buffer;
    Node* closing=current;

    /*
    for (int i=0;i<level_up;i++) {
      closing->end=ptr_buffer;
      closing=closing->parent;
    }
    */

    // move the current point to the closing tag position
    for (int i=0;i<level_up;i++)
        closing=closing->parent;

    Node* lastchild=NULL;
    for (int i=0;i<level_up;i++) {
        lastchild=closing->children[closing->num_children-1];
        
        // reallocate the closing tag's children list 
        closing->children=(Node**)realloc(closing->children,sizeof(Node*)*(closing->num_children+lastchild->num_children));
        // copy all children of the last child as the new children of closing tag
        for (int j=closing->num_children; j<closing->num_children+lastchild->num_children; j++) {
            closing->children[j]=lastchild->children[j-closing->num_children];
            lastchild->children[j-closing->num_children]->parent=closing;
        }

        // update the count of children
        closing->num_children=closing->num_children+lastchild->num_children;

        // update pointer (may have problem)
        // this is to make all sub-brank go up to right below the closing tag leve
        if (lastchild->num_children != 0) {
            lastchild->next_sibling=lastchild->children[0];
            lastchild->end=lastchild->children[0]->start;
            lastchild->num_children=0;
            // just an array, safe to free
            // try uncomment this
            free(lastchild->children);
            lastchild->children=NULL;
        }
    }

    closing->end=ptr_buffer+len;
    current=closing->parent;
    ptr_buffer+=len;

}


int ricBLOGParser::handleDOMNormalString (int id, char* str, int len) {
    strncpy(ptr_buffer,str,len);
    ptr_buffer+=len;
    return 0;
}


#ifdef DEBUG

/**
 * This function is only used when debugging mode is on
 */

int ricBLOGParser::node_seq=0;
void ricBLOGParser::DOMParseDFS (Node* node, int depth) {
    //printf("%d",depth);
    //for (int j=0;j<depth;j++)
    //    printf ("-");
    //printf("%d\n",node->ID);

    // check for errornous pointer
    if (node->end==NULL || node->start==NULL || node->parent==NULL)
        printf ("#node %d is wrong\n",node->node_seq);
    
    // print the .dot file to generate a tree using graphviz
    for (int i=0;i<node->num_children;i++) {
        node->children[i]->node_seq=++node_seq;
        printf ("\t%d -> %d;\n",node->node_seq,node->children[i]->node_seq);
    }

    // DFS traversal
    for (int i=0;i<node->num_children;i++)
        DOMParseDFS(node->children[i], depth+1);
}

#endif


void ricBLOGParser::separateEntries (int max_count, int* index, int max_count_time, int* index_time, int shift, int final_end, struct node* parent) {

    bool needTime=(max_count>max_count_time?false:true);
    int blogindex=0;

    for (int i=1; i<max_count; i++) {
        if (!needTime) {
            blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index[i]]->start-parent->children[index[i-1]]->start+1));
            strncpy(blogentries[blogindex],parent->children[index[i-1]]->start,parent->children[index[i]]->start-parent->children[index[i-1]]->start);
            (blogentries[blogindex])[parent->children[index[i]]->start-parent->children[index[i-1]]->start]='\0';

            timestampentries[blogindex].date_type=parent->children[index[i-1]+shift]->date_type;
            timestampentries[blogindex].year=parent->children[index[i-1]+shift]->year;
            timestampentries[blogindex].month=parent->children[index[i-1]+shift]->month;
            timestampentries[blogindex].day=parent->children[index[i-1]+shift]->day;
            // search for timestamp to be exist within the date
            for (int t=index[i-1];t<index[i];t++) {
                if (parent->children[t]->has_time) {
                    timestampentries[blogindex].time_type=parent->children[t]->time_type;
                    timestampentries[blogindex].hour=parent->children[t]->hour;
                    timestampentries[blogindex].min=parent->children[t]->min;
                }
            }
            blogindex++;
        } else {
            // need to separate between these date entries
            // may be no entry, how to distinguish?
            // need to handle shifting
            // index_time is sorted already
            bool foundTime=false;
            int first=-1,last=-1;
            for (int t=0;t<max_count_time;t++)
                if (index_time[t] >= index[i]) {
                    break;
                } else if (index_time[t] >= index[i-1]) {
                    first=t;
                    break;
                }
            if (first != -1) {
                foundTime=true;
                last=first;
                for (int t=first;t<max_count_time;t++)
                    if (index_time[t] < index[i])
                        last=t;
                    else
                        break;
            }

            if (first==last) {
                // i.e. only 1 time entry detected within the date entry
                // or no time entry detected
                blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index[i]]->start-parent->children[index[i-1]]->start+1));
                strncpy(blogentries[blogindex],parent->children[index[i-1]]->start,parent->children[index[i]]->start-parent->children[index[i-1]]->start);
                (blogentries[blogindex])[parent->children[index[i]]->start-parent->children[index[i-1]]->start]='\0';

                timestampentries[blogindex].date_type=parent->children[index[i-1]+shift]->date_type;
                timestampentries[blogindex].year=parent->children[index[i-1]+shift]->year;
                timestampentries[blogindex].month=parent->children[index[i-1]+shift]->month;
                timestampentries[blogindex].day=parent->children[index[i-1]+shift]->day;
                if (foundTime) {
                    timestampentries[blogindex].time_type=parent->children[index_time[first]]->time_type;
                    timestampentries[blogindex].hour=parent->children[index_time[first]]->hour;
                    timestampentries[blogindex].min=parent->children[index_time[first]]->min;
                }
                // cout << "blogindex: " << blogindex << endl;
                // cout << "length: " << parent->children[index[i]]->start-parent->children[index[i-1]]->start << endl;
                blogindex++;
            } else {
                // more than 1 time entry
                // time_head and time_tial indicate the number of node in between
                // should we deduct one from the time_head?
                // CAUTION
                int time_head=index_time[first]-index[i-1];
                int time_tail=index[i]-index_time[last]-1;
                // might need to use text displacement instead of tag counts

                // cout << "time_head is: " << time_head << " time_tail is: " << time_tail << endl;

                if (time_head > time_tail) {
                    // probably the time is at the ending of each entry, so
                    // tail is used to determine the ending position
                    // keep scanning, till 
                    int new_time_tail=1;
                    for (;new_time_tail<=time_tail;new_time_tail++) {
                        bool ok=true;
                        for (int t=first; t<last; t++) {
                            if ((index_time[t]+new_time_tail >= index_time[t+1]) || (parent->children[index_time[t]+new_time_tail]->ID != parent->children[index_time[t+1]+new_time_tail]->ID)) {
                                ok=false;
                                break;
                            }
                        }
                        if (!ok) break;
                    }
                    time_tail=new_time_tail-1;

                    // now the new_time_tail should be the time_tail
                    // now start putting the entries
                    // first entry
                    blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index_time[first]+time_tail+1]->start-parent->children[index[i-1]]->start+1));
                    strncpy(blogentries[blogindex],parent->children[index[i-1]]->start,parent->children[index_time[first]+time_tail+1]->start-parent->children[index[i-1]]->start);
                    (blogentries[blogindex])[parent->children[index_time[first]+time_tail+1]->start-parent->children[index[i-1]]->start]='\0';
                    timestampentries[blogindex].date_type=parent->children[index[i-1]+shift]->date_type;
                    timestampentries[blogindex].year=parent->children[index[i-1]+shift]->year;
                    timestampentries[blogindex].month=parent->children[index[i-1]+shift]->month;
                    timestampentries[blogindex].day=parent->children[index[i-1]+shift]->day;
                    timestampentries[blogindex].time_type=parent->children[index_time[first]]->time_type;
                    timestampentries[blogindex].hour=parent->children[index_time[first]]->hour;
                    timestampentries[blogindex].min=parent->children[index_time[first]]->min;
                    blogindex++;
                    // loop
                    for (int t=first+1;t<last;t++) {
                        blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index_time[t]+time_tail+1]->start-parent->children[index_time[t-1]+time_tail+1]->start+1));
                        strncpy(blogentries[blogindex],parent->children[index_time[t-1]+time_tail+1]->start,parent->children[index_time[t]+time_tail+1]->start-parent->children[index_time[t-1]+time_tail+1]->start);
                        (blogentries[blogindex])[parent->children[index_time[t]+time_tail+1]->start-parent->children[index_time[t-1]+time_tail+1]->start]='\0';
                        timestampentries[blogindex].date_type=parent->children[index[i-1]+shift]->date_type;
                        timestampentries[blogindex].year=parent->children[index[i-1]+shift]->year;
                        timestampentries[blogindex].month=parent->children[index[i-1]+shift]->month;
                        timestampentries[blogindex].day=parent->children[index[i-1]+shift]->day;
                        timestampentries[blogindex].time_type=parent->children[index_time[t]]->time_type;
                        timestampentries[blogindex].hour=parent->children[index_time[t]]->hour;
                        timestampentries[blogindex].min=parent->children[index_time[t]]->min;
                        blogindex++;
                    }
                    // last entry
                    blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index[i]]->start-parent->children[index_time[last-1]+time_tail+1]->start+1));
                    strncpy(blogentries[blogindex],parent->children[index_time[last-1]+time_tail+1]->start,parent->children[index[i]]->start-parent->children[index_time[last-1]+time_tail+1]->start);
                    (blogentries[blogindex])[parent->children[index[i]]->start-parent->children[index_time[last-1]+time_tail+1]->start]='\0';
                    timestampentries[blogindex].date_type=parent->children[index[i-1]+shift]->date_type;
                    timestampentries[blogindex].year=parent->children[index[i-1]+shift]->year;
                    timestampentries[blogindex].month=parent->children[index[i-1]+shift]->month;
                    timestampentries[blogindex].day=parent->children[index[i-1]+shift]->day;
                    timestampentries[blogindex].time_type=parent->children[index_time[last]]->time_type;
                    timestampentries[blogindex].hour=parent->children[index_time[last]]->hour;
                    timestampentries[blogindex].min=parent->children[index_time[last]]->min;
                    blogindex++;
                } else {
                    // probably the time is at the beginning of each entry since tail > head
                    // head is used to determine the beginning position
                    int new_time_head=1;
                    for (;new_time_head<=time_head;new_time_head++) {
                        bool ok=true;
                        for (int t=first+1; t<=last; t++) {
                            if ((index_time[t]-new_time_head <= index_time[t-1]) || (parent->children[index_time[t]-new_time_head]->ID != parent->children[index_time[t-1]-new_time_head]->ID)) {
                                ok=false;
                                break;
                            }
                        }
                        if (!ok) break;
                    }
                    time_head=new_time_head-1;
                    // now the time_head is the new one
                    // first
                    blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index_time[first+1]-time_head]->start-parent->children[index[i-1]]->start+1));
                    strncpy(blogentries[blogindex],parent->children[index[i-1]]->start,parent->children[index_time[first+1]-time_head]->start-parent->children[index[i-1]]->start);
                    (blogentries[blogindex])[parent->children[index_time[first+1]-time_head]->start-parent->children[index[i-1]]->start]='\0';
                    timestampentries[blogindex].date_type=parent->children[index[i-1]+shift]->date_type;
                    timestampentries[blogindex].year=parent->children[index[i-1]+shift]->year;
                    timestampentries[blogindex].month=parent->children[index[i-1]+shift]->month;
                    timestampentries[blogindex].day=parent->children[index[i-1]+shift]->day;
                    timestampentries[blogindex].time_type=parent->children[index_time[first]]->time_type;
                    timestampentries[blogindex].hour=parent->children[index_time[first]]->hour;
                    timestampentries[blogindex].min=parent->children[index_time[first]]->min;
                    blogindex++;
                    // loop
                    for (int t=first+1;t<last;t++) {
                        blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index_time[t+1]-time_head]->start-parent->children[index_time[t]-time_head]->start+1));
                        strncpy(blogentries[blogindex],parent->children[index_time[t]-time_head]->start,parent->children[index_time[t+1]-time_head]->start-parent->children[index_time[t]-time_head]->start);
                        (blogentries[blogindex])[parent->children[index_time[t+1]-time_head]->start-parent->children[index_time[t]-time_head]->start]='\0';
                        timestampentries[blogindex].date_type=parent->children[index[i-1]+shift]->date_type;
                        timestampentries[blogindex].year=parent->children[index[i-1]+shift]->year;
                        timestampentries[blogindex].month=parent->children[index[i-1]+shift]->month;
                        timestampentries[blogindex].day=parent->children[index[i-1]+shift]->day;

                        timestampentries[blogindex].time_type=parent->children[index_time[t]]->time_type;
                        timestampentries[blogindex].hour=parent->children[index_time[t]]->hour;
                        timestampentries[blogindex].min=parent->children[index_time[t]]->min;
                        blogindex++;
                    }
                    // last entry
                    blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index[i]]->start-parent->children[index_time[last]-time_head]->start+1));
                    strncpy(blogentries[blogindex],parent->children[index_time[last]-time_head]->start,parent->children[index[i]]->start-parent->children[index_time[last]-time_head]->start);
                    (blogentries[blogindex])[parent->children[index[i]]->start-parent->children[index_time[last]-time_head]->start]='\0';
                    timestampentries[blogindex].date_type=parent->children[index[i-1]+shift]->date_type;
                    timestampentries[blogindex].year=parent->children[index[i-1]+shift]->year;
                    timestampentries[blogindex].month=parent->children[index[i-1]+shift]->month;
                    timestampentries[blogindex].day=parent->children[index[i-1]+shift]->day;
                    timestampentries[blogindex].time_type=parent->children[index_time[last]]->time_type;
                    timestampentries[blogindex].hour=parent->children[index_time[last]]->hour;
                    timestampentries[blogindex].min=parent->children[index_time[last]]->min;
                    blogindex++;
                }
            }                                
        }
#ifdef DEBUG2
        printf ("%s <hr>\n",blogentries[i-1]);
#endif
    }

    
    // figure out the final one
    if (!needTime) {
        // only single time entry found in the last day
        blogentries[blogindex]=(char*)malloc(sizeof(char*)*(parent->children[final_end]->end-parent->children[index[max_count-1]]->start+1));
        strncpy(blogentries[blogindex],parent->children[index[max_count-1]]->start,parent->children[final_end]->end-parent->children[index[max_count-1]]->start);
        (blogentries[blogindex])[parent->children[final_end]->end-parent->children[index[max_count-1]]->start]='\0';
        timestampentries[blogindex].date_type=parent->children[index[max_count-1]+shift]->date_type;
        timestampentries[blogindex].year=parent->children[index[max_count-1]+shift]->year;
        timestampentries[blogindex].month=parent->children[index[max_count-1]+shift]->month;
        timestampentries[blogindex].day=parent->children[index[max_count-1]+shift]->day;
        // search for timestamp
        for (int t=index[max_count-1];t<=final_end;t++)
            if (parent->children[t]->has_time) {
                timestampentries[blogindex].time_type=parent->children[t]->time_type;
                timestampentries[blogindex].hour=parent->children[t]->hour;
                timestampentries[blogindex].min=parent->children[t]->min;
            }
        blogindex++;
    } else {
        // may be more than 1 entry on the last day
        // locate the first and last occurence of time entry
        int first=-1,last=-1;
        bool foundTime=false;
        for (int t=0;t<max_count_time;t++)
            if (index_time[t] > final_end) {
                break;
            } else if (index_time[t] >= index[max_count-1]) {
                first=t;
                break;
            }
        if (first != -1) {
            foundTime=true;
            last=first;
            for (int t=first;t<max_count_time;t++)
                if (index_time[t] <= final_end)
                    last=t;
                else
                    break;
        }

        if (first==last) {
            // i.e. only 1 time entry or nothing
            blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[final_end]->end-parent->children[index[max_count-1]]->start+1));
            strncpy(blogentries[blogindex],parent->children[index[max_count-1]]->start,parent->children[final_end]->end-parent->children[index[max_count-1]]->start);
            (blogentries[blogindex])[parent->children[final_end]->end-parent->children[index[max_count-1]]->start]='\0';
            timestampentries[blogindex].date_type=parent->children[index[max_count-1]+shift]->date_type;
            timestampentries[blogindex].year=parent->children[index[max_count-1]+shift]->year;
            timestampentries[blogindex].month=parent->children[index[max_count-1]+shift]->month;
            timestampentries[blogindex].day=parent->children[index[max_count-1]+shift]->day;
            if (foundTime) {
                timestampentries[blogindex].time_type=parent->children[index_time[first]]->time_type;
                timestampentries[blogindex].hour=parent->children[index_time[first]]->hour;
                timestampentries[blogindex].min=parent->children[index_time[first]]->min;
            }
            blogindex++;
        } else {
            // more than 1 time entry
            // time_head and time_tial indicate the number of node in between
            int time_head=index_time[first]-index[max_count-1]-1;
            int time_tail=final_end-index_time[last];
            if (time_head > time_tail) {
                // probably the time is at the ending of each entry
                // tail is used to determine the ending position
                int new_time_tail=1;
                for (;new_time_tail<=time_tail;new_time_tail++) {
                    bool ok=true;
                    for (int t=first; t<last; t++) {
                        if ((index_time[t]+new_time_tail >= index_time[t+1]) || (parent->children[index_time[t]+new_time_tail]->ID != parent->children[index_time[t+1]+new_time_tail]->ID)) {
                            ok=false;
                            break;
                        }
                    }
                    if (!ok) break;
                }
                time_tail=new_time_tail-1;
                // now the new_time_tail should be the time_tail
                // now start putting the entries
                // first entry
                blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index_time[first]+time_tail+1]->start-parent->children[index[max_count-1]]->start+1));
                strncpy(blogentries[blogindex],parent->children[index[max_count-1]]->start,parent->children[index_time[first]+time_tail+1]->start-parent->children[index[max_count-1]]->start);
                (blogentries[blogindex])[parent->children[index_time[first]+time_tail+1]->start-parent->children[index[max_count-1]]->start]='\0';
                timestampentries[blogindex].date_type=parent->children[index[max_count-1]+shift]->date_type;
                timestampentries[blogindex].year=parent->children[index[max_count-1]+shift]->year;
                timestampentries[blogindex].month=parent->children[index[max_count-1]+shift]->month;
                timestampentries[blogindex].day=parent->children[index[max_count-1]+shift]->day;
                timestampentries[blogindex].time_type=parent->children[index_time[first]]->time_type;
                timestampentries[blogindex].hour=parent->children[index_time[first]]->hour;
                timestampentries[blogindex].min=parent->children[index_time[first]]->min;
                blogindex++;
                // loop
                for (int t=first+1;t<last;t++) {
                    blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index_time[t]+time_tail+1]->start-parent->children[index_time[t-1]+time_tail+1]->start+1));
                    strncpy(blogentries[blogindex],parent->children[index_time[t-1]+time_tail+1]->start,parent->children[index_time[t]+time_tail+1]->start-parent->children[index_time[t-1]+time_tail+1]->start);
                    (blogentries[blogindex])[parent->children[index_time[t]+time_tail+1]->start-parent->children[index_time[t-1]+time_tail+1]->start]='\0';
                    timestampentries[blogindex].date_type=parent->children[index[max_count-1]+shift]->date_type;
                    timestampentries[blogindex].year=parent->children[index[max_count-1]+shift]->year;
                    timestampentries[blogindex].month=parent->children[index[max_count-1]+shift]->month;
                    timestampentries[blogindex].day=parent->children[index[max_count-1]+shift]->day;
                    timestampentries[blogindex].time_type=parent->children[index_time[t]]->time_type;
                    timestampentries[blogindex].hour=parent->children[index_time[t]]->hour;
                    timestampentries[blogindex].min=parent->children[index_time[t]]->min;
                    blogindex++;
                }
                // last entry
                blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[final_end]->end-parent->children[index_time[last-1]+time_tail+1]->start+1));
                strncpy(blogentries[blogindex],parent->children[index_time[last-1]+time_tail+1]->start,parent->children[final_end]->end-parent->children[index_time[last-1]+time_tail+1]->start);
                (blogentries[blogindex])[parent->children[final_end]->end-parent->children[index_time[last-1]+time_tail+1]->end]='\0';
                timestampentries[blogindex].date_type=parent->children[index[max_count-1]+shift]->date_type;
                timestampentries[blogindex].year=parent->children[index[max_count-1]+shift]->year;
                timestampentries[blogindex].month=parent->children[index[max_count-1]+shift]->month;
                timestampentries[blogindex].day=parent->children[index[max_count-1]+shift]->day;
                timestampentries[blogindex].time_type=parent->children[index_time[last]]->time_type;
                timestampentries[blogindex].hour=parent->children[index_time[last]]->hour;
                timestampentries[blogindex].min=parent->children[index_time[last]]->min;
                blogindex++;
            } else {
                // probably the time is at the beginning of each entry
                // head is used to determine the beginning position
                int new_time_head=1;
                for (;new_time_head<=time_head;new_time_head++) {
                    bool ok=true;
                    for (int t=first+1; t<=last; t++) {
                        if ((index_time[t]-new_time_head <= index_time[t-1]) || (parent->children[index_time[t]-new_time_head]->ID != parent->children[index_time[t-1]-new_time_head]->ID)) {
                            ok=false;
                            break;
                        }
                    }
                    if (!ok) break;
                }
                time_head=new_time_head-1;
                // now the time_head is the new one
                // first
                blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index_time[first+1]-time_head]->start-parent->children[index[max_count-1]]->start+1));
                strncpy(blogentries[blogindex],parent->children[index[max_count-1]]->start,parent->children[index_time[first+1]-time_head]->start-parent->children[index[max_count-1]]->start);
                (blogentries[blogindex])[parent->children[index_time[first+1]-time_head]->start-parent->children[index[max_count-1]]->start]='\0';
                timestampentries[blogindex].date_type=parent->children[index[max_count-1]+shift]->date_type;
                timestampentries[blogindex].year=parent->children[index[max_count-1]+shift]->year;
                timestampentries[blogindex].month=parent->children[index[max_count-1]+shift]->month;
                timestampentries[blogindex].day=parent->children[index[max_count-1]+shift]->day;
                timestampentries[blogindex].time_type=parent->children[index_time[first]]->time_type;
                timestampentries[blogindex].hour=parent->children[index_time[first]]->hour;
                timestampentries[blogindex].min=parent->children[index_time[first]]->min;
                blogindex++;
                // loop
                for (int t=first+1;t<last;t++) {
                    blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[index_time[t+1]-time_head]->start-parent->children[index_time[t]-time_head]->start+1));
                    strncpy(blogentries[blogindex],parent->children[index_time[t]-time_head]->start,parent->children[index_time[t+1]-time_head]->start-parent->children[index_time[t]-time_head]->start);
                    (blogentries[blogindex])[parent->children[index_time[t+1]-time_head]->start-parent->children[index_time[t]-time_head]->start]='\0';
                    timestampentries[blogindex].date_type=parent->children[index[max_count-1]+shift]->date_type;
                    timestampentries[blogindex].year=parent->children[index[max_count-1]+shift]->year;
                    timestampentries[blogindex].month=parent->children[index[max_count-1]+shift]->month;
                    timestampentries[blogindex].day=parent->children[index[max_count-1]+shift]->day;
                    timestampentries[blogindex].time_type=parent->children[index_time[t]]->time_type;
                    timestampentries[blogindex].hour=parent->children[index_time[t]]->hour;
                    timestampentries[blogindex].min=parent->children[index_time[t]]->min;
                    blogindex++;
                }
                // last
                blogentries[blogindex]=(char*)malloc(sizeof(char)*(parent->children[final_end]->end-parent->children[index_time[last]-time_head]->start+1));
                strncpy(blogentries[blogindex],parent->children[index_time[last]-time_head]->start,parent->children[final_end]->end-parent->children[index_time[last]-time_head]->start);
                (blogentries[blogindex])[parent->children[final_end]->end-parent->children[index_time[last]-time_head]->start]='\0';
                timestampentries[blogindex].date_type=parent->children[index[max_count-1]+shift]->date_type;
                timestampentries[blogindex].year=parent->children[index[max_count-1]+shift]->year;
                timestampentries[blogindex].month=parent->children[index[max_count-1]+shift]->month;
                timestampentries[blogindex].day=parent->children[index[max_count-1]+shift]->day;
                timestampentries[blogindex].time_type=parent->children[index_time[last]]->time_type;
                timestampentries[blogindex].hour=parent->children[index_time[last]]->hour;
                timestampentries[blogindex].min=parent->children[index_time[last]]->min;
                blogindex++;
            }
        }
    }
    if (blogentries_count!=blogindex) {
        cout << "Subtle error occur, recovered" << endl;
        blogentries_count=blogindex;
    }
#ifdef DEBUG2
    printf ("%s",blogentries[max_count-1]);
#endif
}



// this is the function to traverse the tree in BFS manner and search for date
int ricBLOGParser::DOMParseBFS (Node* node) {
    // used to hold the list of nodes to be examined
    queue<Node*> node_list;
    
    int parse_result=0;
    Node* this_node;
    node_list.push(node);

    do {

        this_node=node_list.front();
        node_list.pop();

        for (int i=0; i< this_node->num_children; i++)
            node_list.push(this_node->children[i]);

        if (this_node->num_children > THRESHOLD) {
            // there is a large fan-out, thus worthwhile to investigate

            int childwithdate_count=0;
            int childwithdate_count_between=0;
            int childwithtime_count=0;
            int childwithtime_count_between=0;

            for (int i=0;i<this_node->num_children;i++) {
                // for each children, check existence of date/timestamp
                // printf ("\t it's child is of ID %d\n",this_node->children[i]->ID);
                const char *cstart,*cend;
                cstart=this_node->children[i]->start;
                cend=this_node->children[i]->end;
                const std::string teststring(cstart,0,cend-cstart);
                // check against all date regular expression
                for (int k=0;k<date_re;k++)
                    if (regex_search(teststring.begin(),teststring.end(),match_result,*date_re_engine[k],match_flag)) {
                        this_node->children[i]->year=date_processor[k]->getYear(match_result);
                        this_node->children[i]->month=date_processor[k]->getMonth(match_result);
                        this_node->children[i]->day=date_processor[k]->getDay(match_result);
                        if (isValidDate(this_node->children[i]->year, this_node->children[i]->month, this_node->children[i]->day,k)) {
                            this_node->children[i]->has_date=true;
                            this_node->children[i]->date_type=k+1;
                            childwithdate_count++;
                            break;
                        }
                    }
                // check again all time regular expression
                for (int k=0;k<time_re;k++)
                    if (regex_search(teststring.begin(),teststring.end(),match_result,*time_re_engine[k],match_flag)) {
                        this_node->children[i]->time_type=k+1;
                        this_node->children[i]->hour=time_processor[k]->getHour(match_result);
                        this_node->children[i]->min=time_processor[k]->getMin(match_result);
                        if (isValidTime(this_node->children[i]->hour, this_node->children[i]->min)) {
                            this_node->children[i]->has_time=true;
                            this_node->children[i]->time_type=k+1;
                            childwithtime_count++;
                            break;
                        }
                    }
                
                
                // for each text between the children, check existence of date/timestamp
                if (i==0)
                    cstart=this_node->start;
                else
                    cstart=this_node->children[i-1]->end;
                cend=this_node->children[i]->start;
                const std::string teststring1(cstart, 0, cend-cstart);
                // check against all date regular expression
                for (int k=0;k<date_re;k++)
                    if (regex_search(teststring1.begin(),teststring1.end(),match_result,*date_re_engine[k],match_flag)) {
                        this_node->children[i]->prev_year=date_processor[k]->getYear(match_result);
                        this_node->children[i]->prev_month=date_processor[k]->getMonth(match_result);
                        this_node->children[i]->prev_day=date_processor[k]->getDay(match_result);
                        if (isValidDate(this_node->children[i]->prev_year, this_node->children[i]->prev_month, this_node->children[i]->prev_day,k)) {                            
                            this_node->children[i]->prev_has_date=true;
                            this_node->children[i]->prev_date_type=k+1;
                            childwithdate_count_between++;
                            break;
                        }
                    }
                // check against all time regular expression
                for (int k=0;k<time_re;k++)
                    if (regex_search(teststring1.begin(),teststring1.end(),match_result,*time_re_engine[k],match_flag)) {
                        this_node->children[i]->prev_hour=time_processor[k]->getHour(match_result);
                        this_node->children[i]->prev_min=time_processor[k]->getMin(match_result);
                        if (isValidTime(this_node->children[i]->prev_hour,this_node->children[i]->prev_min)) {
                            this_node->children[i]->prev_has_time=true;
                            this_node->children[i]->prev_time_type=k+1;
                            childwithtime_count_between++;
                            break;
                        }
                    }
                
            }
            
            // handle the portion between last child and ending of the current node
            const std::string teststring2(this_node->children[this_node->num_children-1]->end,0,this_node->end-this_node->children[this_node->num_children-1]->end);
            // check against all date and time regular expression
            for (int k=0;k<date_re;k++)
                if (regex_search(teststring2.begin(),teststring2.end(),match_result,*date_re_engine[k],match_flag)) {
                    this_node->last_year=date_processor[k]->getYear(match_result);
                    this_node->last_month=date_processor[k]->getMonth(match_result);
                    this_node->last_day=date_processor[k]->getDay(match_result);
                    if (isValidDate(this_node->last_year, this_node->last_month, this_node->last_day,k)) {
                        this_node->last_has_date=true;
                        this_node->last_date_type=k+1;
                        childwithdate_count_between++;
                        break;
                    }
                }
            for (int k=0;k<time_re;k++)
                if (regex_search(teststring2.begin(),teststring2.end(),match_result,*time_re_engine[k],match_flag)) {
                    this_node->last_hour=time_processor[k]->getHour(match_result);
                    this_node->last_min=time_processor[k]->getMin(match_result);
                    if (isValidTime(this_node->last_hour, this_node->last_min)) {
                        this_node->last_has_time=true;
                        this_node->last_time_type=k+1;
                        childwithtime_count_between++;
                        break;
                    }
                }


#ifdef DEBUG2
            // print out debug message
            cout << "number of children with date entry: " << childwithdate_count << endl;
            cout << "number of children with date entry (in between): " << childwithdate_count_between << endl;
#endif


            // finish locating the date and time position in the DOM tree and now divide the entries

            if (childwithdate_count >= DATETHRESHOLD || childwithdate_count_between >= DATETHRESHOLD) {
                // date entry is strong enough to separate
                if (childwithdate_count > childwithdate_count_between) {
                    // handle the case where date appear inside the tag

                    // should check for children
                    // check which date type is the majority
                    // we only focus on date entry of the same type
                    // whether majority or minority is still justifiable
                    int *date_type_count=(int*)calloc(MAX_NUM_RE,sizeof(int));
                    int *time_type_count=(int*)calloc(MAX_NUM_RE,sizeof(int));

                    for (int i=0;i<this_node->num_children;i++) {
                        if (this_node->children[i]->has_date) {
                            date_type_count[this_node->children[i]->date_type-1]+=1;
                            //cout << this_node->children[i]->year << "-"
                            //     << this_node->children[i]->month << "-"
                            //     << this_node->children[i]->day << endl;
                        }
                        if (this_node->children[i]->has_time)
                            time_type_count[this_node->children[i]->time_type-1]+=1;
                    }

                    // find the date pattern which occur most and 2nd most abundently
                    int max_count=0;
                    int max_type=-1;
                    for (int i=0;i<date_re;i++) {
                        if (date_type_count[i] > max_count && date_type_count[i] >= DATETHRESHOLD) {
                            max_count = date_type_count[i];
                            max_type = i+1;
                        }
                    }
                    // cout << max_count << " " <<  max_count_2 << endl;
                    // find the time pattern which occur most abundently
                    int max_count_time=0;
                    int max_type_time=-1;
                    for (int i=0;i<time_re;i++)
                        if (time_type_count[i]>max_count_time) {
                            max_count_time=time_type_count[i];
                            max_type_time=i+1;
                        }                    
                    free(date_type_count);
                    free(time_type_count);
                    

                    // if can't find it here, then skip and do BFS again
                    if (max_type == -1)
                        continue;

                    // a constraint to ensure all date originate from the same tag branch
                    int *date_tagid_count=(int*)calloc(ELEM_COUNT,sizeof(int));
                    int *time_tagid_count=(int*)calloc(ELEM_COUNT,sizeof(int));
                    for (int i=0;i<this_node->num_children;i++) {
                        if (this_node->children[i]->date_type == max_type)
                            date_tagid_count[this_node->children[i]->ID]+=1;
                        if (this_node->children[i]->time_type == max_type_time)
                            time_tagid_count[this_node->children[i]->ID]+=1;
                    }

                    max_count=0;
                    int max_tag_id=-1;
                    max_count_time=0;
                    int max_tag_id_time=-1;
                    for (int i=0;i<ELEM_COUNT;i++) {
                        if (date_tagid_count[i] > max_count) {
                            max_count=date_tagid_count[i];
                            max_tag_id=i;
                        }
                        if (time_tagid_count[i] > max_count_time) {
                            max_count_time=time_tagid_count[i];
                            max_tag_id_time=i;
                        }
                    }
                    free (date_tagid_count);
                    free (time_tagid_count);
                    // finish checking this constraint
                    

                    // locate where the date and time entry happen
                    // index is a list containing the index to children branch which contain date entry
                    // NEED TO CHECK
                    // how come static allocation won't give bugs?
                    int *index=(int*)calloc(999,sizeof(int));
                    int *index_time=(int*)calloc(999,sizeof(int));
                    int ind=0;
                    int ind_time=0;

                    for (int i=0;i<this_node->num_children;i++) {
                        // max_type start to count from 1
                        if (this_node->children[i]->date_type == max_type && this_node->children[i]->ID == max_tag_id)
                            index[ind++]=i;
                        if (this_node->children[i]->time_type == max_type_time && this_node->children[i]->ID == max_tag_id_time)
                            index_time[ind_time++]=i;
                    }

                    // cout << "max_type " << max_type << endl;
                    // cout << "max_count " << max_count << endl;

                    // check if the date are aligned reverse chronologically
                    // also take care of the type 10 ambiguity by trying to reverse month and day
                    bool valid_first=true;
                    bool valid_second=true;
                    if (!isValidDate(this_node->children[index[0]]->year,
                                     this_node->children[index[0]]->month,
                                     this_node->children[index[0]]->day,0))
                        valid_first=false;
                    else {
                        for (int i=1;i<max_count;i++) {
                            if (!isDateInOrder(this_node->children[index[i-1]],this_node->children[index[i]]) ||
                                !isValidDate(this_node->children[index[i]]->year,
                                             this_node->children[index[i]]->month,
                                             this_node->children[index[i]]->day,0)) {
                                valid_first=false;
                                break;
                            }
                        }
                    }
                    
                    if (!valid_first && (max_type!=10 || max_type!=11 || max_type!=12)) {
                        free (index);
                        free (index_time);
                        index = NULL;
                        index_time = NULL;
                        continue;
                    }

                    if (max_type==10 || max_type==11 || max_type==12) {
                        int swap;
                        for (int i=0;i<max_count;i++) {
                            swap=this_node->children[index[i]]->month;
                            this_node->children[index[i]]->month=this_node->children[index[i]]->day;
                            this_node->children[index[i]]->day=swap;

                        }
                        if (!isValidDate(this_node->children[index[0]]->year,
                                         this_node->children[index[0]]->month,
                                         this_node->children[index[0]]->day,0))
                            valid_second=false;
                        else {
                            for (int i=1;i<max_count;i++) {
                                if (!isDateInOrder(this_node->children[index[i-1]],this_node->children[index[i]]) ||
                                    !isValidDate(this_node->children[index[i]]->year,
                                                 this_node->children[index[i]]->month,
                                                 this_node->children[index[i]]->day,0)) {
                                    valid_second=false;
                                    break;
                                }
                            }
                        }
                        if (!valid_first && !valid_second) {
                            free(index);
                            free(index_time);
                            index = NULL;
                            index_time = NULL;
                            continue;
                        } else if (valid_first && valid_second) {
                            int last_month=this_node->children[index[0]]->month;
                            int last_day=this_node->children[index[0]]->day;
                            int month_count=0;
                            int day_count=0;
                            for (int i=1;i<max_count;i++) {
                                if (this_node->children[index[i]]->month != last_month) {
                                    last_month=this_node->children[index[i]]->month;
                                    month_count++;
                                }
                                if (this_node->children[index[i]]->day != last_day) {
                                    last_day=this_node->children[index[i]]->day;
                                    day_count++;
                                }
                            }
                            if (month_count > day_count) {
                                for (int i=0;i<max_count;i++) {
                                    swap=this_node->children[index[i]]->month;
                                    this_node->children[index[i]]->month=this_node->children[index[i]]->day;
                                    this_node->children[index[i]]->day=swap;
                                }
                            }
                        } else if (valid_first && !valid_second) {
                            for (int i=0;i<max_count;i++) {
                                swap=this_node->children[index[i]]->month;
                                this_node->children[index[i]]->month=this_node->children[index[i]]->day;
                                this_node->children[index[i]]->day=swap;
                            }
                        }
                    }

                    // finish checking
                    if (max_count < 2) {
                        free(index);
                        free(index_time);
                        index = NULL;
                        index_time = NULL;
                        continue;
                    }

                    // now the index contain all the index to nodes that contain date entry
                    // here we calculate the nodes in between those with date entries
                    // and the number of nodes before the first and after the last one
                    // interval = 0 means there is no sub-branch between each date entry
                    // interval_ind = 0 means all interval are the same, if not, i.e. varying (useful in determining the end of last entry)
                    int head=index[0];
                    int tail=this_node->num_children-1-index[max_count-1];
                    int interval=999;
                    int interval_ind=0;
                    bool even_interval=true;

                    // find the interval between date entries
                    for (int i=1;i<max_count;i++)
                        if (index[i]-index[i-1]-1 < interval) {
                            interval=index[i]-index[i-1]-1;
                            interval_ind=i-1;
                            // case of something smaller found later
                            if (interval_ind)
                                even_interval=false;
                        } else if (index[i]-index[i-1]-1 > interval)
                            // case of something larger found later
                            even_interval=false;

                    

                    // allocate space for storing blogentries
                    // might need to delay the allocation to sometimes later
                    // may be max_count_time, let's do it

                    // CAUTION:
                    // This is very subtle and leads to bugs, (ideally) blogentries_count should be the maximum number of blog entries
                    // However, due to parsing error or unstructureness of weblog, it may not be the case and we need to
                    // pre allocate more memeory max_count+max_count and finalize the number of entries later
                    // in the separateEntries function
                    // is this still useful?
                    blogentries_count=(max_count>max_count_time?max_count:max_count_time);
                    blogentries=(char**)calloc(max_count+max_count_time,sizeof(char*));
                    timestampentries=(Timestamp*)calloc(max_count+max_count_time,sizeof(Timestamp));


                    // debug information
                    // printf ("max_type: %d, max_count:%d\n<br>\n",max_type,max_count);
                    // printf ("max_type_time: %d, max_count_time:%d\n<br>\n",max_type_time,max_count_time);
                    // printf ("head: %d, tail: %d, interval:%d, uniform: %d\n<br>\n",head,tail,interval,(even_interval?1:0));
                    
                    if (interval==0) {
                        // figure out the last entry
                        int final_end=0;
                        if (!even_interval) {
                            // uneven interval, need to figure out the most likely ending tag (heuristic)
                            // the heuristic is to scan from the end until we meet the potential ending tag
                            // the potential_tag can be some other stuff
                            int potential_tag=this_node->children[index[max_count-1]-1]->ID;
                            //int potential_tag=this_node->children[index[interval_ind+1]-1]->ID;
                            
                            for (final_end=this_node->num_children-1; final_end > index[max_count-1]; final_end--)
                                // printf ("<b>WE FOUND IT</b>");
                                if (this_node->children[final_end]->ID==potential_tag)
                                    break;
                        } else {
                            // even interval
                            // so the last entry is believed to be the last node itself
                            final_end=index[max_count-1];
                        }
                        // check if these entries are too small to be believed a postings
                        if (!isBlog(max_count,index,this_node)) {
                            free(blogentries);
                            free(timestampentries);
                            free(index);
                            free(index_time);
                            continue;
                        }
                        // separate according to time
                        separateEntries(max_count,index,max_count_time,index_time,0,final_end,this_node);
                        // free(index);
                        // free(index_time);
                    } else if (head==0) {
                        // no problem, ok, clear
                        // i.e. for sure that date is the beginning node
                        // the last one include the end node till some end (need to figure it out)
                        int final_end=0;
                        if (!even_interval) {
                            // uneven interval, need to figure out the most likely ending tag (heuristic)
                            // the heuristic is to scan from the end until we meet the potential ending tag
                            int potential_tag=this_node->children[index[max_count-1]-1]->ID;
                            for (final_end=this_node->num_children-1; final_end > index[max_count-1]; final_end--)
                                //printf ("<b>WE FOUND IT</b>\n");
                                if (this_node->children[final_end]->ID==potential_tag) {
                                    //printf ("<b>WE FOUDN IT</b>%d\n",this_node->num_children-1-final_end);
                                    break;
                                }
                            
                        } else {
                            // even interval
                            // so the last entry is believed to be the last date entry plus the regular interval
                            final_end=index[max_count-1]+(interval>tail?tail:interval);
                        }
                        // check if these entries are too small to be believed a postings
                        if (!isBlog(max_count,index,this_node)) {
                            free(blogentries);
                            free(timestampentries);
                            free(index);
                            free(index_time);
                            continue;
                        }
                        // separate according to time
                        separateEntries(max_count,index,max_count_time,index_time,0,final_end,this_node);
                        // free(index);
                        // free(index_time);
                    } else {
                        // now, head is not 0, tail may or may not be 0, need to check interval is even or not
                        // i.e. for now the head has something, not sure which one to begin with
                        // a heuristic is to shift the index one by one ahead, stop while the tag IDs are different
                        // interval is not 0, head is not 0, so we should shift at least min(head,interval) to check
                        
                        // to decide whether to shift back or not
                        // a complicated elegant ad-hoc generic algorithm to determine whether we need to shift or not
                        bool need_shifting=false;
                        int final_end;
                        int potential_endtag;
                        
                        if (interval > tail) {
                            // case 1, since ther interval is larger than tail, so we  need
                            // shifting accomodate one entry at the end
                            need_shifting=true;
                        } else {
                            // case 2, interval is smaller than tail, might not need to shift
                            // need more checking
                            if (!even_interval) {
                                // uneven interval
                                // if even can't repeat the smallest interval, then consider shifting
                                // should start matching from the end becuase it is uneven case

                                potential_endtag=this_node->children[index[interval_ind+1]-1]->ID;
                                for (final_end=this_node->num_children-1;final_end>index[max_count-1];final_end--) {
                                    if (this_node->children[final_end]->ID==potential_endtag)
                                        break;
                                }
                                if (final_end-index[max_count-1] < interval)
                                    need_shifting=true;
                                else {
                                    // still have chance to match
                                    for (int i=0;i<interval;i++) {
                                        if (this_node->children[final_end-i]->ID != this_node->children[index[interval_ind+1]-1-i]->ID) {
                                            need_shifting=true;
                                            break;
                                        }
                                    }
                                }
                            } else {
                                // even interval
                                // see if the tail can repeat the interval once, if yes, no need to shift backward
                                // if can repeat, then final_end will stop at the last node?
                                for (final_end=index[max_count-1]+1; final_end<=index[max_count-1]+interval; final_end++) {
                                    if (this_node->children[final_end]->ID != this_node->children[final_end-interval-1]->ID) {
                                        need_shifting=true;
                                        break;
                                    }
                                }
                                final_end--;
                            }                           
                        }
                        
                        // here, need_shifting indicate we need to shift back or not, 
                        // and final_end contain the index to the end of the last entry
                        
                        int max_shift=interval>head?head:interval;
                        int shift=0;
                        if (need_shifting) {
                            bool inconsistent=false;
                            do {
                                for (int k=1;k<max_count;k++) {
                                    if (this_node->children[index[0]-shift-1]->ID != this_node->children[index[k]-shift-1]->ID) {
                                        inconsistent=true;
                                        break;
                                    }
                                }
                                if (!inconsistent)
                                    shift++;
                            } while (shift<max_shift && !inconsistent);
                        }
                        // shift containt the displacement we need to subtract
                        // printf ("shift is %d\n",shift);

                        // why not subtract it from the index first?
                        for (int i=0;i<max_count;i++)
                            index[i]=index[i]-shift;
                        //for (int i=0;i<max_count_time;i++)
                        //    index_time[i]=index[i]-shift;
                        
                        // how about the last one?
                        if (need_shifting) {
                            int potential_tag=this_node->children[index[max_count-1]-1]->ID;
                            if (!even_interval) {
                                for (final_end=this_node->num_children-1;final_end>index[max_count-1];final_end--)
                                    if (this_node->children[final_end]->ID==potential_tag)
                                        break;
                            } else {
                                final_end=index[max_count-1]+(interval>tail?tail:interval);
                            }
                        }
                        // check if these entries are too small to be believed a postings
                        if (!isBlog(max_count,index,this_node)) {
                            free(blogentries);
                            free(timestampentries);
                            free(index);
                            free(index_time);
                            continue;
                        }
                        separateEntries(max_count,index,max_count_time,index_time,shift,final_end,this_node);
                        // free(index);
                        // free(index_time);
                    }    
                    //cout << "before return BFS parse" << endl;
                    //node_list.clear();
                    parse_result=1;
                    break;
                } else {
                    // should check for internal
                    // do we care?
                    parse_result=2;
                    break;
                }
            } else if (childwithtime_count >= DATETHRESHOLD || childwithtime_count_between >= DATETHRESHOLD) {
                if (childwithtime_count > childwithtime_count_between) {
                    // only time, is that ok? do we care?
                    parse_result=3;
                    break;
                } else {
                    parse_result=4;
                    break;
                }
            }
        }

    } while (!node_list.empty());
    
    // clean up
    while (!node_list.empty())
         node_list.pop();


    return parse_result;
}

// check entries based on size
bool ricBLOGParser::isBlog(int max_count, int* index, struct node* parent) {
  return true;
  int size=0;
  int ii;
  for (ii=1; ii<max_count; ii++) {
    size+=parent->children[index[ii]]->start-parent->children[index[ii-1]]->start;
  }
  if (size/ii > NORMAL_ENTRY_SIZE)
      return true;
  else
      return false;
}

// a simple check on date (just to avoid some of the loop holes in regular expression
bool ricBLOGParser::isValidDate (int year, int month, int day, int date_type) {
    if (year>2005 || year < 1995)
        return false;
    if (day<1 || month <1)
        return false;
    switch (month) {
    case 1:
    case 3:
    case 5:
    case 7:
    case 8:
    case 10:
    case 12:
        if (day > 31) return false;
        break;
    case 4:
    case 6:
    case 9:
    case 11:
        if (day > 30) return false;
        break;
    case 2:
        if (day > 29) return false;
        break;
    default:
        // case 9 need to swap month,day
        if (date_type!=9 || date_type!=10 || date_type!=11) {
            return false;
        } else {
            switch (day) {
            case 1:
            case 3:
            case 5:
            case 7:
            case 8:
            case 10:
            case 12:
                if (month > 31) return false;
                break;
            case 4:
            case 6:
            case 9:
            case 11:
                if (month > 30) return false;
                break;
            case 2:
                if (month > 29) return false;
                break;
            default:
                return false;
                break;
            }
            return true;
        }
        break;
    }
    return true;
}

bool ricBLOGParser::isValidTime (int hour, int minute) {
    if (hour < 0 || hour > 23 || minute < 0 || minute > 59)
        return false;
}


void ricBLOGParser::initializeDateRE () {

    match_flag=boost::match_default;
    date_re=14;
    time_re=4;
    for (int i=0;i<date_re;i++)
        date_processor[i]=new ricDateExtractorImpl(i);
    for (int i=0;i<time_re;i++)
        time_processor[i]=new ricTimeExtractorImpl(i);

    // the regular expression for date
    // 1 - weekday
    // 2 - month
    // 3 - day
    // 4 - year
    date_re_engine[0]=new boost::regex("(Monday|Mon|Tuesday|Tue|Wednesday|Wed|Thursday|Thu|Thur|Friday|Fri|Saturday|Sat|Sunday|Sun)\\W{1,3}(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Sept|Oct|Nov|Dec|January|February|March|April|May|June|July|August|September|October|November|December)\\W{1,3}(\\d{1,2})\\W{1,2}(\\d{2,4})",boost::regex::normal|boost::regex::icase);


    // 1 - day
    // 2 - dummy
    // 3 - month
    // 4 - year
    date_re_engine[1]=new boost::regex("(\\d{2})(th|st|nd|rd)\\W+(January|February|March|April|May|June|July|August|September|October|November|December)\\W{1,3}(\\d{4})",boost::regex::normal|boost::regex::icase);


    // 1 - weekday
    // 2 - day
    // 3 - month
    // 4 - year
    date_re_engine[2]=new boost::regex("(Monday|Mon|Tuesday|Tue|Wednesday|Wed|Thursday|Thu|Thur|Friday|Fri|Saturday|Sat|Sunday|Sun)\\W{1,3}(\\d{1,2})\\W{1,3}(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Sept|Oct|Nov|Dec|January|February|March|April|May|June|July|August|September|October|November|December)\\W{1,3}(\\d{2,4})",boost::regex::normal|boost::regex::icase);

    // weird (slashdot and onegoodmove)
    // 1 - weekday
    // 2 - month
    // 3 - day
    date_re_engine[3]=new boost::regex("(Monday|Mon|Tuesday|Tue|Wednesday|Wed|Thursday|Thu|Thur|Friday|Fri|Saturday|Sat|Sunday|Sun)\\W{1,3}(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Sept|Oct|Nov|Dec|January|February|March|April|May|June|July|August|September|October|November|December)\\W{1,4}(\\d{1,2})",boost::regex::normal|boost::regex::icase);

    // this can be bogus
    // 1 - month
    // 2 - day
    // 3 - year
    date_re_engine[4]=new boost::regex("(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Sept|Oct|Nov|Dec|January|Feburary|March|April|May|June|July|August|September|October|November|December).{1,3}?(\\d{1,2})\\W{1,3}?(\\d{2,4})",boost::regex::normal|boost::regex::icase);

    // 1 - weekday
    // 2 - month (numeric)
    // 3 - day (numeric)
    // 4 - year (numeric)
    date_re_engine[5]=new boost::regex("(Monday|Mon|Tuesday|Tue|Wednesday|Wed|Thursday|Thu|Thur|Friday|Fri|Saturday|Sat|Sunday|Sun)\\W{1,3}(\\d{2})-(\\d{2})-(\\d{2,4})",boost::regex::normal|boost::regex::icase);

    // 1 - month
    // 2 - day
    // 3 - year
    date_re_engine[6]=new boost::regex("(January|February|March|April|May|June|July|August|September|October|November|December)\\W{1,3}(\\d{1,2})\\W{1,3}(\\d{4})",boost::regex::normal|boost::regex::icase);

    // 1 - month
    // 2 - day
    // 3 - dummy
    // 4 - year
    date_re_engine[7]=new boost::regex("(January|February|March|April|May|June|July|August|September|October|November|December)\\W{1,3}(\\d{1,2})(th|rd|st|nd)\\W{1,3}(\\d{4})",boost::regex::normal|boost::regex::icase);
    
    // 1 - year
    // 2 - month
    // 3 - day
    date_re_engine[8]=new boost::regex(">[^<]*?(\\d{4})[-|/|\\.](\\d{1,2})[-|/|\\.](\\d{1,2})");

    // 1 - day (month)
    // 2 - month (day)
    // 3 - year
    date_re_engine[9]=new boost::regex(">[^<]*?(\\d{1,2})/(\\d{1,2})/(\\d{4}|\\d{2})");
    date_re_engine[10]=new boost::regex(">[^<]*?(\\d{1,2})-(\\d{1,2})-(\\d{4}|\\d{2})");
    date_re_engine[11]=new boost::regex(">[^>]*?(\\d{1,2})\\.(\\d{1,2})\\.(\\d{4}|\\d{2})");

    // 1 - day
    // 2 - month
    // 3 - year
    date_re_engine[12]=new boost::regex("(\\d{2})\\W{1,3}(January|February|March|April|May|June|July|August|September|October|November|December|Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\\W{1,2}(\\d{2,4})",boost::regex::normal|boost::regex::icase);

    // 1 - day
    // 2 - month
    // 3 - year
    date_re_engine[13]=new boost::regex("(\\d{2})(January|February|March|April|May|June|July|August|September|October|November|December|Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)(\\d{2,4})",boost::regex::normal|boost::regex::icase);



    // now the regex for time, tricky
    // 1 - hour
    // 2 - minute
    // 3 - am/pm    
    time_re_engine[0]=new boost::regex("(\\d{1,2}):(\\d{1,2}):\\d{1,2}\\s?([a|p]m)",boost::regex::normal|boost::regex::icase);
    time_re_engine[1]=new boost::regex("(\\d{1,2}):(\\d{1,2})\\s?([a|p]m)",boost::regex::normal|boost::regex::icase);
    time_re_engine[2]=new boost::regex("@\\s?(\\d{1,2}):(\\d{1,2})\\s?([a|p]m)?",boost::regex::normal|boost::regex::icase);
    time_re_engine[3]=new boost::regex("(\\d{2}):(\\d{2})",boost::regex::normal|boost::regex::icase);

}


/**
 * print the entries in RSS format given the output stream object
 */
void ricBLOGParser::printRSS (const string& name, char* original_blog_text, int num_entries, char** blog_text, Timestamp* blog_time, std::ostream& output) {

    queue<string> links;

    output << "<?xml version=\"1.0\"?>" << endl;
    output << "<rss version=\"0.91\">" << endl;
    output << "<channel>" << endl;
    
    // find the title this blog (use the buffer to search)
    string blogTitle;
    if (findBlogTitle(string(original_blog_text,0,strlen(original_blog_text)), blogTitle)) {
        output << "<title>" << blogTitle << "</title>" << endl;
    } else {
      // otherwise use domain name to be title?
        string::size_type end=name.find("/",8);
        if (end==string::npos) {
            output << "<title>" << name.substr(8) << "</title>" << endl;
        } else {
            output << "<title>" << name.substr(8,end-8) << "</title>" << endl;
        }
        
    }

    output << "<link>" << name << "</link>" << endl;
    output << "<description>Automatically generated RSS from weblog</description>" << endl;

    for (int i=0; i<num_entries; i++) {
        output << "<item>" << endl;
        // find title of this entry, if not found, don't make up something, just let it be
        string entryTitle;
        if (findEntryTitle(string(blog_text[i],0,strlen(blog_text[i])), entryTitle)) {
            output << "<title>" << entryTitle << "</title>" << endl;
        } else {
            output << "<title> </title>" << endl;
        }

        // find links related to this entry
        findLinks(std::string(blog_text[i],0,strlen(blog_text[i])),links);
        if (links.empty()) {
            output << "<link>" << name << "</link>" << endl;
        } else {
            while (!links.empty()) {
                output << "<link>" << links.front() << "</link>" << endl;
                links.pop();
            }
        }
        
        output << "<description>" << blog_text[i] << "</description>" << endl;
        output << "<dc:date>" << blog_time[i].year <<  "-" << blog_time[i].month << "-" << blog_time[i].day << "T" << blog_time[i].hour << ":" << blog_time[i].min << "+00:00</dc:date>" << endl;
        output << "</item>" << endl;
    }

    output << "</channel>" << endl;
    output << "</rss>" << endl;
}


/**
 * given a string, check for all existence of <a href=" ">
 */
void ricBLOGParser::findLinks (const string& input, queue<string>& links) {
    //queue<string> links;
    boost::regex hyperlinkRE("<\\s*?a[^>]+?href\\s*?=\\s*?[\"|']?(http://[^>^\"^']+?)[\"|'|\\s|>]",boost::regex::normal|boost::regex::icase);
    boost::match_results<std::string::const_iterator> link_result;
    
    std::string::const_iterator start=input.begin();

    while(regex_search(start, input.end(), link_result, hyperlinkRE, match_flag)) { 
        // link_result[0] contains the whole string 
        // link_result[1] contains the URL 
        // add parsed URL to the queue
        links.push(std::string(link_result[1].first, link_result[1].second)); 

        // update search position: 
        start = link_result[0].second;
    }
}

/**
 * given a string, check if there can someting to act a title of a posting, just hacking
 */
bool ricBLOGParser::findEntryTitle (const string& input, string& output) {
    // CAUTION
    return false;
    // find first occurenc of these stuff and treat them as title of the entries
    boost::regex titleRE1("<b>([^>^<]+?)</b>",boost::regex::normal|boost::regex::icase);
    boost::regex titleRE2("<h\\d>([^>^<]+?)</h\\d>",boost::regex::normal|boost::regex::icase);
    // boost::regex titleRE3("<span[^>]*>([^>^<]+?)</span>",boost::regex::normal|boost::regex::icase);

    boost::match_results<string::const_iterator> result;


    if (regex_search(input.begin(), input.end(), result, titleRE1, match_flag)) {
        output=std::string(result[1].first,result[1].second);
        return true;
    }
    
    if (regex_search(input.begin(), input.end(), result, titleRE2, match_flag)) {
        output=std::string(result[1].first,result[1].second);
        return true;
    }

    // if (regex_search(input.begin(), input.end(), result, titleRE3, match_flag)) {
    //     output=std::string(result[1].first,result[1].second);
    //     return true;
    // }

    return false;
}

/**
 * given a string, try to see if there is title in it
 */
bool ricBLOGParser::findBlogTitle (const string& input, string& output) {
    // CAUTION
    boost::regex titleRE("<title>([^>^<]+?)</title>",boost::regex::normal|boost::regex::icase);
    
    boost::match_results<string::const_iterator> result;

    if (regex_search(input.begin(), input.end(), result, titleRE, match_flag)) {
        output=std::string(result[1].first,result[1].second);
        return true;
    }

    return false;
}


/**
 * a fucntion to add new regular expression for date
 */
void ricBLOGParser::addDateRE (char *RE, ricDateExtractorInterface* processor) {
  if (date_re < MAX_NUM_RE ) {
    date_re_engine[date_re]=new boost::regex(RE,boost::regex::normal|boost::regex::icase);
    date_processor[date_re++]=processor;
  }
}

/**
 * a function to add new regular expression for time
 */
void ricBLOGParser::addTimeRE (char *RE, ricTimeExtractorInterface* processor) {
  if (time_re < MAX_NUM_RE ) {
    time_re_engine[time_re]=new boost::regex(RE,boost::regex::normal|boost::regex::icase);
    time_processor[time_re++]=processor;
  }
}



bool ricBLOGParser::isDateInOrder (struct node* prev, struct node* next) {
    if (prev->year < next->year)
        return false;
    else if (prev->year == next->year && prev->month < next->month)
        return false;
    else if (prev->year == next->year && prev->month == next->month && prev->day < next->day)
        return false;
    else
        return true;
}

