//
// ricBLOGParser.h
//
// @author Ka Cheung Sia {kcsia@cs.ucla.edu}
// @date   02/01/2005
// @copyright Ka Cheung Sia
//
// Implementation of a parser for blog
//
//
// Copyright (c) 2005, Ka Cheung Sia {kcsia@cs.ucla.edu}
// University of California, Los Angeles
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of the University of California Los Angeles nor
// the names of its contributors may be used to endorse or promote
// products derived from this software without specific prior written
// permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include <boost/regex.hpp>
#include <queue>
#include <iostream>
#include <string>
#include <stack>
#include "ricExtractorInterface.h"

#ifndef __RIC_BLOGPARSER_H
#define __RIC_BLOGPARSER_H

#define MAX_FILE_SIZE 1024*1024*3
// this is ok
#define THRESHOLD 3
// might need to change to 2
#define DATETHRESHOLD 3
// the number of regular expression
#define MAX_NUM_RE 30
// the threshold to define a blog entry
#define NORMAL_ENTRY_SIZE 250

namespace kcsia {
    
    typedef struct {
        int date_type;
        int time_type;
        int year;
        int month;
        int day;
        int hour;
        int min;
    } Timestamp;
    
    typedef struct node {
        int ID;                      // tag id of this node
        char* start;                 // the starting position of this node in the buffer
        char* end;                   // the ending position of this node in the buffer
        int num_children;            // the number of child nodes of this node
        bool has_date;               // to indicate whether this node has date entry or not
        bool prev_has_date;          // to indicate whether the text in front of this node has date entry or not
        bool last_has_date;          // to indicate whether the last portion of this node has date entry or not
        
        bool has_time;
        bool prev_has_time;
        bool last_has_time;
        
        int date_type;               // to indicate what type of date entry this node contains
        int prev_date_type;          // to indicate what type of date entry the text in front of this node contains
        int last_date_type;          // to indicate what type of date entry the text in the last portion of this node contains
        
        int time_type;               // seems of no use other than debug
        int prev_time_type;
        int last_time_type;
        
        struct node** children;      // pointers to all children
        struct node* parent;         // pointer  to parent
        struct node* next_sibling;   // pointer  to next sibling
        
        // only valid when the has_date flag is on
        int year;
        int month;
        int day;
        int hour;
        int min;
        // only valid when the prev_has_date flag is on
        int prev_year;
        int prev_month;
        int prev_day;
        int prev_hour;
        int prev_min;
        // only valid when the last_has_date flag is on
        int last_year;
        int last_month;
        int last_day;
        int last_hour;
        int last_min;
        
        int node_seq;

    } Node;
    
    
    class ricBLOGParser {
        
    private:    
        // static memeber function as callback function of HTML Parser
        static int ricOpenTagHandler (int, char*, int);
        static int ricCloseTagHandler (int, char*, int);
        static int ricTokenHandler (int, char*, int);
        static int ricCommentHandler (int, char*, int);
        static int ricPunctHandler (int, char*, int);
        static int ricUnmatchedHandler (int, char*, int);
        static int ricEntityHandler (int);
        
        static int handleDOMOpenTag (int, char*, int);
        static int handleDOMCloseTag (int, char*, int, int);
        static int handleDOMNormalString (int, char*, int);
        
        
    private:
        // void printEntries ();
        void freeDOM (struct node*);
        int DOMParseBFS (struct node*);    
        void separateEntries (int max_count, int* index, int max_count_time, int* index_time, int shift, int final_end, struct node* parent);
        void initializeDateRE();

        bool isBlog(int max_count, int* index, struct node* parent);
        bool isValidDate(int year, int month, int day, int date_type);
        bool isValidTime(int hour, int minute);
        bool isDateInOrder(struct node* prev, struct node* next);
        
        
    private:
        static Node root;
        static Node* current;
        static stack<int> tag_stack;
        
        static char buffer[MAX_FILE_SIZE];
        static char *ptr_buffer;
        static int instance_count;
        
        boost::match_results<std::string::const_iterator> match_result;
        boost::match_flag_type match_flag;
        boost::regex* date_re_engine[MAX_NUM_RE];
        boost::regex* time_re_engine[MAX_NUM_RE];
        ricDateExtractorInterface* date_processor[MAX_NUM_RE];
        ricTimeExtractorInterface* time_processor[MAX_NUM_RE];
        
        int date_re;
        int time_re;
        
        char** blogentries;
        Timestamp* timestampentries;
        int blogentries_count;
        
    public:
        // constructor
        ricBLOGParser ();
        // destructor
        ~ricBLOGParser ();
        
        // a function to separate a HTML page to parts
        // return value # of entries found
        int parseBlog(char* input, char*** blog_text, Timestamp** blog_time);
        
        // a function to output the parsed stuff from parseBlog to RSS format
        void printRSS (const string& URL, char* original_blog_text, int num_entries, char** blog_text, Timestamp* blog_time, ostream& output=cout);
        
        // two functions for user to add more regular expression to parse date
        // should implemenet this as call back function
        void addDateRE (char* RE, ricDateExtractorInterface* processor);
        void addTimeRE (char* RE, ricTimeExtractorInterface* processor);

        // helper function in finding necessary informaiton in the input string
        void findLinks (const string& input, queue<string>& links);
        bool findEntryTitle (const string& input, string& output);
        bool findBlogTitle (const string& input, string& output);
        
    };

}

#endif





